CHARACTER_FIGURES = {
    "eily": [
        "robot-girl-3123483124.svg",
    ],
    "boy": [
        "boy-stand-arms-fistup-leg-step-blinkgrin-anime",
        "boy-stand-arms-open-leg-open-question-anime", 
        "boy-stand-arms-straight-leg-straddle-calm-anime", 
        "boy-stand-arms-straight-leg-straight-blinkgrin-anime", 
        "boy-stand-arms-straight-leg-straight-blinksay-anime", 
        "boy-stand-arms-straight-leg-straight-complain-anime", 
        "boy-stand-arms-straight-leg-straight-groan-anime", 
        "boy-stand-arms-straight-leg-straight-say-anime", 
        "boy-stand-arms-straight-leg-straight-smilesay-anime", 
        "boy-stand-left-finger1up-right-straight-leg-crossing-blinkgrin-anime", 
        "boy-stand-left-finger1up-right-straight-leg-step-blinkgrin-anime", 
        "boy-stand-left-finger1up-right-straight-leg-step-groan-anime", 
        "boy-stand-left-finger1up-right-straight-leg-straddle-calm-anime", 
        "boy-stand-left-fistup-right-straight-leg-straddle-blinkgrin-anime", 
        "boy-stand-left-fistup-right-straight-leg-straddle-complain-anime", 
        "boy-stand-left-fistup-right-straight-leg-straddle-groan-anime", 
        "boy-stand-left-fistup-right-straight-leg-straddle-smilesay-anime", 
        "boy-stand-left-straight-right-handonhip-leg-straddle-calm-anime", 
        "boy-stand-left-thumbup-right-handonhip-leg-straddle-smilesay-anime", 
        "boy-stand-left-thumbup-right-handonhip-leg-straddle-winking-anime", 
        "boy-stand-left-thumbup-right-handonhip-leg-straight-calm-anime", 
        "boy-standright-left-handonhip-right-thumbup-leg-straddle-winking-anime", 
        "boy-standright-left-straight-right-finger1up-leg-straddle-winking-anime", 
        "boy-standright-left-straight-right-fistup-leg-straddle-smilesay-anime", 
        "boy-standright-left-straight-right-fistup-leg-straddle-winking-anime", 
        "boy-half-left-handonhip-right-finger1up-calm-anime", 
        "boy-half-left-handonhip-right-open-smilesay-anime", 
        "boy-half-left-handonhip-right-thumbup-winking-anime", 
        "boy-head-smilesay-anime", 
        "boy-standright-left-straight-right-finger1up-leg-crossing-blinkgrin-anime", 
        "boy-stand-arms-straight-leg-straddle-smilesay-anime", 
        "boy-stand-arms-straight-leg-straddle-groan-anime", 
        "boy-stand-arms-straight-leg-straddle-blinkgrin-anime", 
        "boy-stand-left-fistup-right-straight-leg-step-groan-anime", 
        "boy-stand-left-fistup-right-straight-leg-step-blinkgrin-anime", 
        "boy-stand-left-fistup-right-straight-leg-crossing-blinkgrin-anime", 
        "boy-half-left-handonhip-right-thumbup-smilesay-anime", 
        "boy-half-arms-straight-winking-anime", 
        "boy-stand-left-straight-right-spear-leg-straddle-khoikhoimask-anime", 
        "boy-stand-left-fistup-right-spear-leg-straddle-khoikhoimask-anime", 
        "boy-stand-left-finger1up-right-straight-leg-crossing-calm-anime"
    ], 
    "girl": [
        "girl-stand-arms-fistup-leg-open-blinkgrin-anime", 
        "girl-stand-arms-handonhip-leg-straight-calm-anime", 
        "girl-stand-arms-handonhip-leg-straight-complain-anime", 
        "girl-stand-arms-handonhip-leg-straight-smilesay-anime", 
        "girl-stand-arms-handonhip-leg-straight-winking-anime", 
        "girl-stand-arms-open-leg-straight-bemused-anime", 
        "girl-stand-arms-open-leg-straight-blinkgrin-anime", 
        "girl-stand-arms-open-leg-straight-question-anime", 
        "girl-stand-arms-open-leg-straight-smilesay-anime", 
        "girl-stand-arms-straight-leg-straight-calm-anime", 
        "girl-stand-left-handonhip-right-thumbup-leg-open-winking-anime", 
        "girl-stand-left-straight-right-finger1up-leg-crossing-question-anime", 
        "girl-stand-left-straight-right-finger1up-leg-straight-groan-anime", 
        "girl-stand-left-straight-right-finger1up-leg-straight-smilesay-anime", 
        "girl-stand-left-straight-right-finger1up-leg-straight-winking-anime", 
        "girl-stand-left-handonhip-right-open-leg-ontiptoe-groan-anime", 
        "girl-half-left-handonhip-right-finger1up-smilesay-anime", 
        "girl-half-left-handonhip-right-open-smilesay-anime", 
        "girl-half-left-handonhip-right-thumbup-winking-anime", 
        "girl-head-smilesay-anime", 
        "girl-stand-left-shield-right-open-leg-straight-khoikhoimask-anime", 
        "girl-stand-left-shield-right-finger1up-leg-straight-khoikhoimask-anime", 
        "girl-stand-arms-handonhip-leg-straight-groan-captaincap-anime", 
        "girl-stand-left-open-right-finger1up-leg-straight-smilesay-captaincap-anime", 
        "girl-stand-left-open-right-finger1up-leg-straight-winking-captaincap-anime", 
        "girl-stand-arms-open-leg-straight-captaincap-anime"
    ]
}